package dev.svitan.plugins

import io.ktor.http.HttpStatusCode
import io.ktor.server.application.Application
import io.ktor.server.application.install
import io.ktor.server.plugins.NotFoundException
import io.ktor.server.plugins.statuspages.StatusPages
import io.ktor.server.response.respond

class AuthorizationException(message: String) : Exception(message)

fun Application.configureErrors() {
    install(StatusPages) {
        exception<Throwable> { call, cause ->
            when (cause) {
                is IllegalArgumentException -> call.respond(
                    status = HttpStatusCode.BadRequest,
                    message = cause.message!!
                )

                is NotFoundException -> call.respond(
                    status = HttpStatusCode.NotFound,
                    message = cause.message ?: ""
                )

                is AuthorizationException -> call.respond(
                    status = HttpStatusCode.Unauthorized,
                    message = cause.message ?: ""
                )

                else -> call.respond(
                    status = HttpStatusCode.InternalServerError,
                    message = cause.message ?: ""
                )
            }
        }
    }
}
