package dev.svitan.services

import io.github.cdimascio.dotenv.Dotenv
import java.io.File
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.UUID

class ExecutorService {
    companion object {
        var dir: File? = null

        fun init(dotenv: Dotenv) {
            val path = dotenv["DATA_DIR"] ?: throw Exception("DATA_DIR not found")
            dir = File(path)

            if (!dir!!.exists()) {
                val created = dir!!.mkdirs()
                if (!created) throw Exception("Unable to create data directory")
            } else if (dir!!.isFile) {
                throw Exception("DATA_DIR must be a directory")
            } else if (!dir!!.canRead() || !dir!!.canWrite() || !dir!!.canExecute()) {
                throw Exception("DATA_DIR must be readable, writable, and executable")
            }
        }

        fun writeFile(uuid: UUID, content: String) {
            if (dir == null) throw Exception("FilesystemService not initialized")

            val file = File(dir, "$uuid.sh")
            file.writeText(content)
        }

        fun deleteFile(uuid: UUID) {
            if (dir == null) throw Exception("FilesystemService not initialized")

            val file = File(dir, "$uuid.sh")
            if (!file.exists()) throw Exception("File not found")

            val deleted = file.delete()
            if (!deleted) throw Exception("Unable to delete file")
        }

        fun executeFile(uuid: UUID) {
            if (dir == null) throw Exception("FilesystemService not initialized")

            val file = File(dir, "$uuid.sh")
            if (!file.exists() || !file.canExecute()) throw Exception("File not found or not executable")

            val now = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
            val stdout = File(dir, "$uuid-$now-stdout")
            val stderr = File(dir, "$uuid-$now-stderr")
            ProcessBuilder("bash", file.absolutePath)
                .redirectOutput(ProcessBuilder.Redirect.to(stdout))
                .redirectError(ProcessBuilder.Redirect.to(stderr))
                .start()
        }

        fun executeSource(uuid: UUID, source: String) {
            if (dir == null) throw Exception("FilesystemService not initialized")

            val now = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
            val stdout = File(dir, "$uuid-$now-stdout")
            val stderr = File(dir, "$uuid-$now-stderr")
            ProcessBuilder("bash", "-c", source)
                .redirectOutput(ProcessBuilder.Redirect.to(stdout))
                .redirectError(ProcessBuilder.Redirect.to(stderr))
                .start()
        }
    }
}
